﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace DMAPI
{
    /// <summary>
    /// Example class for implementation of DMAPI commands using the provided Connect class
    /// </summary>
    class Commands
    {
        private Connect conn;

        public string uid;

        public float account_balance;

        // Store for last list columns
        public string[] columns;

        // Stores the available TLDs after a successful login
        private ArrayList avail_tlds = new ArrayList();

        ///  <summary>Login</summary>
        ///  <param name="dmapiurl">DMAPI base url (for example: https://dmapi.joker.com)</param>
        ///  <param name="username">Username</param>
        ///  <param name="password">Password</param>
        public bool Login(string dmapiurl, string username, string password)
        {
            if (conn != null) return false;

            Response resp;

            conn = new Connect(dmapiurl);
            try
            {
                resp = conn.login(username, password);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                conn = null;
                return false;
            }

            if (resp != null)
            {
                // fetch some additional header fields (case-sensitive)
                this.uid = resp.GetHeaderVar("uid");
                this.account_balance = float.Parse(resp.GetHeaderVar("Account-balance"), CultureInfo.InvariantCulture.NumberFormat);

                // fetch the response body with the available TLDs
                ArrayList result = resp.GetResponseList(true);
                avail_tlds.Clear();
                if (result.Count > 0)
                {
                    foreach (String[] value in result)
                    {
                        avail_tlds.Add(value[0]);
                    }
                    avail_tlds.Sort();
                }

                return true;
            }
            conn = null;
            return false;

        }

        /// <summary>
        /// Login-Status
        /// </summary>
        /// <returns>Returns the current login status (true/false)</returns>
        public bool isConnected()
        {
            return conn != null && conn.isLoggedIn();
        }


        ///  <summary>Logout</summary>
        public void Logout()
        {
            if (conn!=null)
            {
                conn.logout();
                conn = null;
            }
            
        }

        ///  <summary>Returns a the DMAPI Version as string</summary>
        public string GetDmapiVersion()
        {
            Hashtable fields = new Hashtable();
            Response response = null;

            try
            {
                response = conn.execute_request("version", fields);
            } 
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";
            }

            ArrayList result = response.GetResponseList(true);

            if (result.Count > 0)
            {
                foreach (String[] value in result)
                {
                    if (value[0] == "Version")
                    {
                        return value[1];
                    }
                }
            }

            return "";
        }

        ///  <summary>Returns a the available TLDs as string[]</summary>
        public string[] GetAvailableTLDs() 
        {
            if (this.avail_tlds != null && this.avail_tlds.Count > 0)
            {
                return (string[])this.avail_tlds.ToArray(typeof(string)) as string[];
            }
            return null;
        }

        ///  <summary>Returns the owned domains as ArrayList of string[] filtert by pattern string </summary>
        ///  <param name="pattern">pattern string (* = wildcard)</param>
        public ArrayList DomainList(string pattern)
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("pattern",pattern);
            fields.Add("showstatus","1");

            // DMAPI Response
            Response response = null;

            try {
                response = conn.execute_request("query-domain-list", fields);
            } 
            catch (DMAPIException e)
            {
               MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
               return null;
            }
            return response.GetResponseList();

        }

        ///  <summary>Returns a list of request result statuses </summary>
        
        public ArrayList ResultList()
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("pending", "1");
            fields.Add("showall", "1");

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("result-list", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return response.GetResponseList();

        }


        ///  <summary>Returns the whois information for the domain as ArrayList of string[]</summary>
        ///  <param name="domain">domainname</param>
        public ArrayList DomainWhois(string domain)
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("domain", domain);

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("query-whois", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return response.GetResponseList(true);

        }

        ///  <summary>Returns a list of all domain owner emails which are pending verification </summary>
        public ArrayList EmailList()
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("email-list", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            this.columns = response.GetColumns();
            return response.GetResponseList();

        }

        ///  <summary>Returns confirmation detail list for a verification key </summary>
        ///  <param name="key">verification key</param>
        public ArrayList EmailDetails(string key)
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("key", key);

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("email-details", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            this.columns = response.GetColumns();
            return response.GetResponseList();

        }

        ///  <summary>Resends verification email</summary>
        ///  <param name="email">owner email</param>
        public string EmailValidate(string email)
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("email", email);

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("email-validate", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return response.GetHeaderVar("Result");

        }

        ///  <summary>Verify owner email</summary>
        ///  <param name="key">verification key</param>
        ///  <param name="answer">yes/no</param>
        public string EmailVerify(string key, bool answer)
        {
            // Parameters for Http-DMAPI-Request
            Hashtable fields = new Hashtable();
            fields.Add("key", key);
            fields.Add("answer", answer?"yes":"no");

            // DMAPI Response
            Response response = null;

            try
            {
                response = conn.execute_request("email-verify", fields);
            }
            catch (DMAPIException e)
            {
                MessageBox.Show(e.Status + " : " + e.Message, e.Command, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            return response.GetHeaderVar("Result");

        }


    }
}
