﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Net;
using System.Text;
using System.Collections;
using System.Web;

namespace DMAPI
{

    class Connect
    {
        // Stores the DMAPI Base-Url
        private string url;

        // The current auth-id
        private string session ="";

        // The last query url-string
        private string http_query;

        // The available requests after a successful login
        private ArrayList dmapi_avail_requests = new ArrayList();

        // Last http response code
        private string last_http_response_code;

        // Value for empty-fields
        private const string empty_field_value = "[empty]";


        ///  <summary>Constructor</summary>
        ///  <param name="url">DMAPI base url ( for example: https://dmapi.joker.com )</param>
        public Connect(string url)
        {
            this.url = url;
        }

        ///  <summary>Login Procedure</summary>
        ///  <param name="user">username/email</param>
        ///  <param name="passw">password</param>
        ///  <returns>DMAPI Response</returns>
        public Response login(string user, string passw)
        {
            Hashtable fields = new Hashtable();
            fields.Add("username", user);
            fields.Add("password", passw);

            Response response = null;

            response = this.execute_request("login", fields);
            
            session = response.GetHeaderVar("Auth-Sid");

            if (session == "")
            {
                throw new DMAPIException(response.GetHeaderVar("Status-Code"), "login", response.GetHeaderVar("Status-Text"));
            }

            //list of available requests
            this.dmapi_avail_requests = this.get_request_list();

            
            return response;
        }

        public void logout()
        {
            if (session == "") return;
            Response response = null;
            response = this.execute_request("logout", null);
            session = "";
        }

        ///  <summary>Returns an ArrayList(string) of all available requests</summary> 
        private ArrayList get_request_list()
        {
            Hashtable fields = new Hashtable();
            Response response = null;
            ArrayList list = new ArrayList();
            response = this.execute_request("query-request-list", fields);

            ArrayList result = response.GetResponseList(true);
            if (result.Count > 0)
            {
                foreach (String[] value in result)
                {
                    list.Add(value[0]);
                }
            }
            
            return list;
        }

        /// <summary>
        /// Establishing a HTTPWeb connection. Returns the HTTP response.
        /// </summary>
        /// <param name="query">Complete DMAPI Command Url</param>
        /// <returns>HTTP response</returns>
        private string query_host(string query)
        {
            // used to build entire input
            StringBuilder sb = new StringBuilder();

            // used on each read operation
            byte[] buf = new byte[8192];

            // prepare the web page we will be asking for
            HttpWebRequest request = (HttpWebRequest)
                WebRequest.Create(this.url + query);

            // execute the request

            HttpWebResponse response;

            try
            {
                response = (HttpWebResponse)request.GetResponse();
            }
            catch (WebException e)
            {
                response = (HttpWebResponse)e.Response;
            }

            // we will read data via the response stream
            Stream resStream = response.GetResponseStream();

            this.last_http_response_code = ((int)response.StatusCode).ToString();

            string tempString = null;
            int count = 0;

            do
            {
                // fill the buffer with data
                count = resStream.Read(buf, 0, buf.Length);

                // make sure we read some data
                if (count != 0)
                {
                    // translate from bytes to ASCII text
                    tempString = Encoding.UTF8.GetString(buf, 0, count);

                    // continue building the string
                    sb.Append(tempString);
                }
            }
            while (count > 0); // any more data to read?

            // print out page source
            return sb.ToString();
        }


        /// <summary>Prepares the request to be sent, submits it and checks the DMAPI response.</summary>
        /// <param name="request">which request should be executed</param>
        /// <param name="parameters">array containing the request parameteres</param>
        /// <param name="response">HTTP response (by reference)</param>
        /// <retval name="TRUE">if successfull</retval>
        public Response execute_request(string request, Hashtable parameters)
        {
            Response response = null;
            if (this.is_request_available(request))
            {
                //build the query
                this.assemble_query(request, parameters);

                //send the request
                string raw_res = this.query_host(this.http_query);

                try
                {
                    response = new Response(raw_res);
                }
                catch (ParseException e)
                {
                    throw new DMAPIException("I", request, e.Message);
                }

                //status
                if (last_http_response_code != null && last_http_response_code.StartsWith("2"))
                {
                    return response;
                }
                else
                {
                    if ("401" == last_http_response_code)
                    {
                        //deletes session auth-id
                        this.session = "";
                    }
                    throw new DMAPIException(response.GetHeaderVar("Status-Code"), request, response.GetHeaderVar("Error"));
                }
            }
            else
            {
                throw new DMAPIException("e", request , "execute_request(): Request '"+request+"' is not supported in this version of DMAPI.");
            }
        }

       

        private bool is_request_available(string request)
        {
            if (request == "login" || request == "query-request-list")
            {
                return true;
            }
            foreach (string item in dmapi_avail_requests)
            {
                if (request == item)
                {
                    return true;
                }
            }
            return false;

        }


        /// <summary>
        /// Intermediate function to prepare the HTTP requests
        /// </summary>
        /// <param name="request">DMAPI specific request</param>
        /// <param name="parameters">contains all required requet parameters</param>
        private void assemble_query(string request, Hashtable parameters)
        {
            this.http_query = "/request/" + request + "?" + this.http_build_query(parameters);
        }

        /// <summary>
        /// Builds an HTTP query from a given set of parameters.
        /// </summary>
        /// <param name="formdata">GET field values</param>
        /// <returns>GET parameter string</returns>
        private string http_build_query(Hashtable formdata)
        {
            if (formdata == null) formdata = new Hashtable();
            if (session != null && session != "")
            {
                formdata.Add("auth-sid", session);
            }

            //The IP of the user should be always present in the requests
            formdata.Add("client-ip", this.getLocalIP());

            // If the array is empty, return null
            if (formdata.Count == 0)
            {
                return null;
            }

            // Start building the query
            ArrayList tmp = new ArrayList();
            foreach (DictionaryEntry de in formdata)
            {
                string val = de.Value.ToString();

                if (val.Trim() != "")
                {
                    if (val.ToLower() == empty_field_value)
                    {
                        val = "";
                    }

                    val = HttpUtility.UrlEncode(de.Key.ToString()) + '=' + HttpUtility.UrlEncode(val.Trim());

                    tmp.Add(val);
                }
            }
            string http_request = String.Join("&", tmp.ToArray(typeof(string)) as string[]);
            return http_request;
        }

        /// <summary>
        /// Retrieves the local IP
        /// </summary>
        /// <returns>ip string</returns>
        private string getLocalIP()
        {
            IPHostEntry host;
            string localIP = "";
            host = Dns.GetHostEntry(Dns.GetHostName());
            foreach (IPAddress ip in host.AddressList)
            {
                if (ip.AddressFamily.ToString() == "InterNetwork")
                {
                    localIP = ip.ToString();
                }
            }
            return localIP;
        }

        /// <summary>
        /// Login-Status
        /// </summary>
        /// <returns>Returns the current login status (true/false)</returns>
        public bool isLoggedIn()
        {
            return session!=null && session != "";
        }
    }

}
