﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;

namespace DMAPI
{
    class Response
    {
        private Hashtable response_header;
        private string response_body;


        /// <summary>
        /// Parses the HTTP response body from the DMAPI server
        /// </summary>
        /// <param name="response">Response Body</param>
        public Response(string response)
        {
            string[] raw_arr = response.Trim().Split("\n\n", 2);

            if (raw_arr.Length > 0)
            {
                if (1 == raw_arr.Length)
                {
                    this.SetHeader(raw_arr[0]);

                }
                else if (2 == raw_arr.Length)
                {
                    this.SetHeader(raw_arr[0]);
                    this.response_body = raw_arr[1];
                }
            }
            else
            {
                throw new ParseException("Couldn't parse the response into response header and response body\nRaw result:\n" + response);
            }
        }


        public string GetHeaderVar(string key)
        {
            if (response_header != null && response_header.ContainsKey(key))
            {
                return response_header[key].ToString();
            }

            return "";

        }

        /// <summary>
        /// Sets the header variables by parsing the header string of the DMAPI response.
        /// </summary>
        /// <param name="header">DMAPI header as text</param>
        public void SetHeader(string header)
        {

            Hashtable result = new Hashtable();

            Regex r = new Regex("^(?<key>[^\\s]+):\\s+(?<value>.+)\\s*$", RegexOptions.Multiline);

            // Find matches.
            MatchCollection matches = r.Matches(header);

            foreach (Match match in matches)
            {
                if (!result.Contains(match.Groups["key"].Value))
                    result.Add(match.Groups["key"].Value, match.Groups["value"].Value);
            }

            if (matches.Count == 0)
            {
                throw new ParseException("Header line not parseable - pattern do not match\nRaw header:\n" + header);
            }

            this.response_header = result;
        }

        public void SetResponseBody(string body)
        {
            this.response_body = body;
        }

        public string GetResponseBody()
        {
            return this.response_body;
        }

        ///  <summary>Returns the column names of a list as string[]</summary>
        ///  <retval>Returns a string[] of column names, if available</retval>
        public string[] GetColumns()
        {
            string cols = this.GetHeaderVar("Columns");
            if (cols != "") return cols.Split(',');
            return null;
        }

        ///  <summary>Returns the parsed body-message as a ArrayList(string[])</summary>
        ///  <retval>Returns an ArrayList(string[]) of all body lines</retval>
        public ArrayList GetResponseList()
        {
            return this.GetResponseList(false);
        }

        ///  <summary>Returns the parsed body-message as a ArrayList(string[])</summary>
        ///  <param name="keyval">TRUE for key->value format or FALSE list format</param>
        ///  <returns>ArrayList(string[]) of all body lines</retval>
        public ArrayList GetResponseList(bool keyval)
        {
            ArrayList result = new ArrayList();
            if (this.response_body == null) return result;
            char separator = ' ';
            switch (this.GetHeaderVar("Separator"))
            {
                case "TAB":
                    separator = '\t';
                    break;
                default:
                    separator = ' ';
                    break;
            }
            string text = this.response_body;
            text = text.Trim();
            if (text != "")
            {
                string[] raw_arr = text.Split('\n');
                if (raw_arr.Length > 0)
                {
                    foreach (string value in raw_arr)
                    {
                        if (!keyval)
                        {
                            result.Add(value.Split(separator));
                        }
                        else
                        {
                            string[] temp = value.Split(" ", 2);
                            temp[0] = temp[0].TrimEnd(':');
                            result.Add(temp);

                        }
                    }
                }
            }
            return result;
        }


        ///  <summary>Returns the parsed body-message (key->value) as a Hashtable(string->string)</summary>
        ///  <returns>Hashtable(string->string) of all key->value body lines</retval>
        public Hashtable GetResponseValues()
        {
            Hashtable result = new Hashtable();
            if (this.response_body == null) return result;
            string text = this.response_body;
            text = text.Trim();
            if (text != "")
            {
                string[] raw_arr = text.Split('\n');
                if (raw_arr.Length > 0)
                {
                    foreach (string value in raw_arr)
                    {
                            string[] temp = value.Split(" ", 2);
                            temp[0] = temp[0].TrimEnd(':');
                            result.Add(temp[0], temp[1]);

                    }
                }
            }
            return result;
        }

    }

    /// <summary>
    /// Extensions for the .NET string class
    /// </summary>
    static class StringExtensions
    {
        /// <summary>
        /// Returns an array of strings, each of which is a substring of string formed by splitting it on boundaries formed by the string delimiter. 
        /// </summary>
        /// <param name="value"></param>
        /// <param name="matcher"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        /// <remarks>Like php explode function</remarks>
        public static string[] Split(this string value, string delimiter, int limit)
        {

            string[] temp = value.Split(new string[] { delimiter }, StringSplitOptions.None);
            string[] result = new string[limit < temp.Length ? limit : temp.Length];
            for (int i = 0; i < result.Length; i++)
            {
                result[i] = temp[i];
            }
            if (temp.Length > limit) result[limit - 1] = String.Join(delimiter, temp, limit - 1, temp.Length - limit + 1);
            return result;
        }
    }

}
